/* svg2ps - Render an SVG image to a PS image (using cairo)
 *
 * Copyright  2002 USC/Information Sciences Institute
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of
 * Information Sciences Institute not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.  Information Sciences Institute
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * INFORMATION SCIENCES INSTITUTE DISCLAIMS ALL WARRANTIES WITH REGARD
 * TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL INFORMATION SCIENCES
 * INSTITUTE BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth <cworth@isi.edu>
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <svg-cairo.h>

#include "args.h"

#define PS_UNITS_PER_INCH	72.0
#define X_PPI			300.0
#define Y_PPI			300.0

static svg_cairo_status_t
render_to_ps (FILE *svg_file, FILE *ps_file, double scale, int width, int height);

int 
main (int argc, char **argv)
{
    args_t args;
    args_parse (&args, argc, argv);
    FILE *svg_file, *ps_file;
    svg_cairo_status_t status;

    if (strcmp (args.svg_filename, "-") == 0) {
	svg_file = stdin;
    } else {
	svg_file = fopen (args.svg_filename, "r");
	if (svg_file == NULL) {
	    fprintf (stderr, "%s: failed to open %s: %s\n",
		     argv[0], args.svg_filename, strerror(errno));
	    return 1;
	}
    }

    if (strcmp (args.ps_filename, "-") == 0) {
	ps_file = stdout;
    } else {
	ps_file = fopen (args.ps_filename, "w");
	if (ps_file == NULL) {
	    fprintf (stderr, "%s: failed to open %s: %s\n",
		     argv[0], args.ps_filename, strerror (errno));
	    return 1;
	}
    }
	       
    status = render_to_ps (svg_file, ps_file, args.scale, args.width, args.height);
    if (status) {
	fprintf (stderr, "%s: failed to render %s\n", argv[0], args.svg_filename);
	return 1;
    }

    return 0;
}

static svg_cairo_status_t
render_to_ps (FILE *svg_file, FILE *ps_file, double scale, int width, int height)
{
    svg_cairo_status_t status;
    cairo_t *cr;
    svg_cairo_t *svgc;

    cr = cairo_create ();

    status = svg_cairo_create (&svgc);
    if (status) {
	fprintf (stderr, "Failed to create svg_cairo_t. Exiting.\n");
	exit(1);
    }

    status = svg_cairo_parse_file (svgc, svg_file);
    if (status)
	return status;

    if (width < 0 || height < 0) {
	svg_cairo_get_size (svgc, &width, &height);
	width = (width * scale + 0.5);
	height = (height * scale + 0.5);
    }

    cairo_scale (cr, scale, scale);

    cairo_set_target_ps (cr, ps_file,
		width/PS_UNITS_PER_INCH, height/PS_UNITS_PER_INCH,
		X_PPI, Y_PPI);

    /* XXX: This probably doesn't need to be here (eventually) */
    cairo_set_rgb_color (cr, 1, 1, 1);

    svg_cairo_render (svgc, cr);

    cairo_show_page (cr);

    svg_cairo_destroy (svgc);

    cairo_destroy (cr);

    return SVG_CAIRO_STATUS_SUCCESS;
}
